﻿using ProjectGasMachine.Drawnings;
using ProjectGasMachine.MovementStrategy;


namespace ProjectGasMachine;
public partial class FormGasMachine : Form
{
    /// <summary>
    /// поле-объект для прорисовки объекта
    /// </summary>
    private DrawningMachine? _drawningMachine;

    /// <summary>
    /// стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    /// <summary>
    /// конструктор формы
    /// </summary>
    public FormGasMachine()
    {
        InitializeComponent();
        _strategy = null;
    }


    /// <summary>
    /// метод прорисовки машины
    /// </summary>
    private void Draw()
    {
        if (_drawningMachine == null)
        {
            return;
        }


        Bitmap bmp = new(pictureBoxGasMachine.Width, pictureBoxGasMachine.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningMachine.DrawTransport(gr);
        pictureBoxGasMachine.Image = bmp;
    }
    /// <summary>
    /// создание объекта класса-перемещение
    /// </summary>
    /// <param name="type"></param>
    private void CreateObject(string type)
    {
        Random random = new();
        switch (type)
        {
            case nameof(DrawningMachine):
                _drawningMachine = new DrawningMachine(random.Next(100, 300), random.Next(1000, 3000),
                    Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)));
                break;
            case nameof(DrawningGasMachine):
                _drawningMachine = new DrawningGasMachine(random.Next(100, 300), random.Next(1000, 3000),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Color.FromArgb(random.Next(0, 256), random.Next(0, 256), random.Next(0, 256), random.Next(0, 256)),
            Convert.ToBoolean(random.Next(0, 2)), Convert.ToBoolean(random.Next(0, 2)));
                break;
            default:
                return;
        }


        _drawningMachine.SetPictureSize(pictureBoxGasMachine.Width, pictureBoxGasMachine.Height);
        _drawningMachine.SetPosition(random.Next(10, 100), random.Next(10, 100));
        _strategy = null;
        comboBoxStrategy.Enabled = true;
        Draw();
    }






    /// <summary>
    /// обработка нажатия кнопки "создать газовоз"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonCreate_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningGasMachine));


    /// <summary>
    /// обработка нажатия кнопки "создать орнамент на колесах"
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>

    private void ButtonCreateMachine_Click(object sender, EventArgs e) => CreateObject(nameof(DrawningMachine));



    /// <summary>
    /// перемещение объекта по форме
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningMachine == null)
        {
            return;
        }
        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningMachine.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningMachine.MoveTransport(DirectionType.Down);
                break;
            case "buttonRight":
                result = _drawningMachine.MoveTransport(DirectionType.Right);
                break;
            case "buttonLeft":
                result = _drawningMachine.MoveTransport(DirectionType.Left);
                break;
        }
        if (result)
        {
            Draw();
        }
    }

    private void ButtonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningMachine == null) 
        {
            return;
        }

        if (comboBoxStrategy.Enabled) 
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };

            if (_strategy == null) 
            {
                return;
            }

            _strategy.SetData(new MoveableMachine(_drawningMachine), pictureBoxGasMachine.Width, pictureBoxGasMachine.Height);     
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish) 
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }




    }
}

