﻿using ProjectGasMachine.Drawnings;
using System.Reflection.PortableExecutable;

namespace ProjectGasMachine.MovementStrategy;

/// <summary>
/// класс-реализация IMoveableObjects с использованием DrawningMachine
/// </summary>
public class MoveableMachine : IMoveableObjects
{
    /// <summary>
    /// поле-объект класса DrawningMachine или его наследника
    /// </summary>
    private readonly DrawningMachine? _machine = null;


    /// <summary>
    /// конструктор
    /// </summary>
    /// <param name="machine">объект класса DrawningMachine</param>
    public MoveableMachine(DrawningMachine machine)
    {
        _machine = machine;
    }

    public ObjectParameters? GetObjectPosition 
    {
        get 
        {
            if (_machine == null || _machine.EntityGas == null || !_machine.GetPosX.HasValue || !_machine.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_machine.GetPosX.Value, _machine.GetPosY.Value, _machine.GetWidth, _machine.GetHeight);
        }
    }





    public int GetStep => (int)(_machine?.EntityGas?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_machine == null || _machine.EntityGas == null) 
        {
            return false;
        }
        return _machine.MoveTransport(GetDirectionType(direction));
    }

    /// <summary>
    /// конвертация из MovementDirection в DirectionType
    /// </summary>
    /// <param name="direction">MovementDirection</param>
    /// <returns>DirectionType</returns>
    private static DirectionType GetDirectionType(MovementDirection direction) 
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }

}
